import { __decorate, __metadata, __param } from "tslib";
import BetterSSE from 'better-sse';
import { createHmac } from 'crypto';
import { IncomingMessage, ServerResponse } from 'http';
import { Body, Controller, Ctx, Get, HeaderParam, NotAcceptableError, OnUndefined, Param, Post, Req, Res, UnauthorizedError } from 'routing-controllers';
import { OpenAPI } from 'routing-controllers-openapi';
let HookController = class HookController {
    channel = BetterSSE.createChannel();
    async bootSSE(request, response, state) {
        if (!request.headers.accept?.includes('text/event-stream'))
            throw new NotAcceptableError();
        const session = await BetterSSE.createSession(request, response, { state });
        this.channel.register(session);
    }
    /**
     * @param  name       `X-GitHub-Event`: Event name
     * @param  id         `X-GitHub-Delivery`: Unique ID for this delivery
     * @param  signature  `X-Hub-Signature`: HMAC hex digest of the payload
     *
     * @see {@link https://docs.github.com/webhooks/webhook-events-and-payloads#delivery-headers}
     */
    emitEvent(name = '', id = 0, signature = '', event) {
        const { GITHUB_HOOK_SECRET = '' } = process.env, data = JSON.stringify(event), [algorithm, signValue] = signature?.split('=') || [];
        const checkValue = algorithm &&
            createHmac(algorithm, GITHUB_HOOK_SECRET)
                .update(data)
                .digest('hex');
        event = { ...event, id, signature };
        if (algorithm && signValue !== checkValue) {
            /**
             * Signature verification failed
             *
             * @event external:EventEmitter#error
             *
             * @type {object}
             *
             * @property {string}   id    `X-GitHub-Delivery` header
             * @property {string[]} sign  Array splited from
             *                            `X-Hub-Signature` header
             *
             * @see {@link https://developer.github.com/webhooks/#payloads}
             */
            this.channel.emit('error', event);
            throw new UnauthorizedError('Signature verification failed');
        }
        this.channel.broadcast(event, name, {
            filter: ({ state: { event: type, owner, repository } }) => (!type || type === event.event) &&
                (!owner || owner === event.repository?.owner.login) &&
                (!repository || repository === event.repository?.name)
        });
    }
    getOrgEvent(owner, request, response, { req, res } = {}) {
        return this.bootSSE(req || request, res || response, { owner });
    }
    getRepoEvent(owner, repository, request, response, { req, res } = {}) {
        return this.bootSSE(req || request, res || response, {
            owner,
            repository
        });
    }
    getIssueEvent(owner, repository, request, response, { req, res } = {}) {
        return this.bootSSE(req || request, res || response, {
            owner,
            repository,
            event: 'issues'
        });
    }
};
__decorate([
    OpenAPI({ description: 'Receive all kinds of Event from a Web Hook' }),
    Post('/hookHub'),
    OnUndefined(204),
    __param(0, HeaderParam('X-GitHub-Event')),
    __param(1, HeaderParam('X-GitHub-Delivery')),
    __param(2, HeaderParam('X-Hub-Signature')),
    __param(3, Body()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Object, Object]),
    __metadata("design:returntype", void 0)
], HookController.prototype, "emitEvent", null);
__decorate([
    OpenAPI({
        description: 'Organization SSE'
    }),
    Get('/orgs/:org/events'),
    OnUndefined(204),
    __param(0, Param('org')),
    __param(1, Req()),
    __param(2, Res()),
    __param(3, Ctx()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, IncomingMessage,
        ServerResponse, Object]),
    __metadata("design:returntype", void 0)
], HookController.prototype, "getOrgEvent", null);
__decorate([
    OpenAPI({
        description: 'Repository SSE'
    }),
    Get('/repos/:owner/:repo/events'),
    __param(0, Param('owner')),
    __param(1, Param('repo')),
    __param(2, Req()),
    __param(3, Res()),
    __param(4, Ctx()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, IncomingMessage,
        ServerResponse, Object]),
    __metadata("design:returntype", void 0)
], HookController.prototype, "getRepoEvent", null);
__decorate([
    OpenAPI({
        description: 'Repository issue SSE'
    }),
    Get('/repos/:owner/:repo/issues/events'),
    __param(0, Param('owner')),
    __param(1, Param('repo')),
    __param(2, Req()),
    __param(3, Res()),
    __param(4, Ctx()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, IncomingMessage,
        ServerResponse, Object]),
    __metadata("design:returntype", void 0)
], HookController.prototype, "getIssueEvent", null);
HookController = __decorate([
    Controller()
], HookController);
export { HookController };
//# sourceMappingURL=Hook.js.map